﻿/************************************************************************************************************
 * 
 *                  Microchip Generic USB DLL Interface Class
 *              
 *          Interfaces to mpusbapi.dll
 * 
 *          Adapted from - Mat Clayton -   mat@piccoder.co.uk -  www.PICcoder.co.uk
 *          
 *          C# Class - James E. McCullers 2010
 *              
 ***********************************************************************************************************/
using System;
using System.Runtime.InteropServices;

using DWORD = System.UInt32;
using HANDLE = System.UInt32;

namespace MPUSBAPI 
{
    public class MPUSB
    {
        #region  Class Definitions

        public enum PipeDir : uint
        {
            Write = 0,
            Read = 1
        }

        public const int FAIL = 0;
        public const int SUCCESS = 1;
        #endregion

        string      vidpid;
        UInt32      outPipe;     
        UInt32      inPipe;
        string      outPipeEP;
        string      inPipeEP; 

        #region Imported DLL functions from mpusbapi.dll

        [DllImportAttribute("mpusbapi.dll")]
        public static extern DWORD MPUSBGetDLLVersion();

        [DllImportAttribute("mpusbapi.dll")]
        public static extern int MPUSBGetDeviceCount(string pVID_PID);

        [DllImportAttribute("mpusbapi.dll")]
        public static extern DWORD MPUSBOpen(DWORD instance,
                                             [MarshalAs(UnmanagedType.LPStr)] string pVID_PID,
                                             [MarshalAs(UnmanagedType.LPStr)] string pEP,
                                             [MarshalAs(UnmanagedType.U4)] PipeDir dwDir,
                                             DWORD dwReserved);

        [DllImportAttribute("mpusbapi.dll")]
        public static extern DWORD MPUSBRead(HANDLE handle,
                                             [MarshalAs(UnmanagedType.LPArray)] byte[] pData,
                                             DWORD dwLen,
                                             out DWORD pLength,
                                             DWORD dwMilliseconds);    

        [DllImport("mpusbapi.dll", EntryPoint = "MPUSBClose")]
        public static extern bool MPUSBClose(HANDLE handle);
        #endregion

        // Class 
        public MPUSB() { }

        #region Properties
        public uint DLLVersion
        {
            get { return MPUSBGetDLLVersion(); }
        }

        public int DeviceCount
        {
            get { return MPUSBGetDeviceCount(vidpid); }
        }

        public string VidPid
        {
            get { return vidpid; }
            set { vidpid = value; }
        }

        public string InPipe
        {
            get { return inPipeEP; }
            set { inPipeEP = value; }
        }

        public string OutPipe
        {
            get { return outPipeEP; }
            set { outPipeEP = value; }
        }

        public Int32 InStatus
        {
            get { return (Int32)inPipe; }
        }
        
        public Int32 OutStatus
        {
            get { return (Int32)outPipe; }
        }
        #endregion

        #region Methods
        public void OpenPipe(PipeDir pipe)
        {
            if (pipe == PipeDir.Read)
                inPipe = MPUSBOpen(0, vidpid, inPipeEP, PipeDir.Read, 0);
            else 
                outPipe = MPUSBOpen(0, vidpid, outPipeEP, PipeDir.Write, 0);
        }

        public void ClosePipe(PipeDir pipe)
        {
            if (pipe == PipeDir.Read)
                MPUSBClose((HANDLE)inPipe);
            else
                MPUSBClose((HANDLE)outPipe);
        }

        public DWORD Read(byte[] pData,
                          DWORD dwLen,
                          out DWORD pLength,
                          DWORD dwMilliseconds)
        { 
            return MPUSBRead(inPipe, pData, dwLen, out pLength, dwMilliseconds);
        }
        #endregion
    }
}

